@@ -37,7 +37,7 @@
                 i       irate scalar
                 k       krate scalar
                 a       arate vector
-                f       fregency variable
+                f       frequency variable
                 w       spectral variable
                 x       krate scalar or arate vector
                 S       String
@@ -57,6 +57,7 @@
                 j              "             "         -1
                 h              "             "        127
                 O       optional k-rate, defaulting to  0
+                J              "             "         -1
                 V              "             "          .5
                 P              "             "          1
                 y       begins indef list of aargs (any count)
@@ -70,6 +71,7 @@
                 s       arate or krate scalar
                 X       multiple args (a, k, or i-rate)     IV - Sep 1 2002
                 N       multiple args (a, k, i, or S-rate)
+                F       multiple args (f-rate)
    (these types must agree with rdorch.c)                               */
 
 /* If dsblksize is 0xffff then translate on output arg
@@ -133,12 +135,12 @@ OENTRY opcodlst_1[] = {
 { "ihold",  S(LINK),    1,      "",     "",     ihold                   },
 { "turnoff",S(LINK),    2,      "",     "",     NULL,   turnoff         },
 { "=.r",    S(ASSIGN),  1,      "r",    "i",    rassign                 },
-{ "=.i",    S(ASSIGN),  1,      "i",    "i",    assign                  },
-{ "=.k",    S(ASSIGN),  2,      "k",    "k",    NULL,   assign          },
+{ "=.i",    S(ASSIGNM), 1,      "IIIIIIIIIIIIIIIIIIIIIIII", "m", minit  },
+{ "=.k",    S(ASSIGNM), 2,      "zzzzzzzzzzzzzzzzzzzzzzzz", "z", NULL, minit },
 { "=.a",    S(ASSIGN),  4,      "a",    "x",    NULL,   NULL,   aassign },
-{ "init.i",  S(ASSIGN), 1,      "i",    "i",    init                    },
-{ "init.k",  S(ASSIGN), 1,      "k",    "i",    init                    },
-{ "init.a",  S(ASSIGN), 1,      "a",    "i",    ainit                   },
+{ "init.i", S(ASSIGNM), 1,      "IIIIIIIIIIIIIIIIIIIIIIII", "m", minit  },
+{ "init.k", S(ASSIGNM), 1,      "zzzzzzzzzzzzzzzzzzzzzzzz", "m", minit  },
+{ "init.a", S(ASSIGNM), 1,      "mmmmmmmmmmmmmmmmmmmmmmmm", "m", mainit },
 { ">",      S(RELAT),   0,      "B",    "kk",   gt,     gt              },
 { ">=",     S(RELAT),   0,      "B",    "kk",   ge,     ge              },
 { "<",      S(RELAT),   0,      "B",    "kk",   lt,     lt              },
@@ -261,8 +263,9 @@ OENTRY opcodlst_1[] = {
 { "ftsr.i",S(EVAL),     1,      "i",    "i",    ftsr                    },
 { "ftlptim.i",S(EVAL),  1,      "i",    "i",    ftlptim                 },
 { "ftchnls.i",S(EVAL),  1,      "i",    "i",    ftchnls                 },
-{ "i.k",   S(ASSIGN),   1,      "i",    "k",    init                    },
-{ "k.i",   S(ASSIGN),   1,      "k",    "i",    init                    },
+{ "ftcps.i",S(EVAL),    1,      "i",    "i",    ftcps                   },
+{ "i.k",   S(ASSIGN),   1,      "i",    "k",    assign                  },
+{ "k.i",   S(ASSIGN),   1,      "k",    "i",    assign                  },
 { "cpsoct.i",S(EVAL),   1,      "i",    "i",    cpsoct                  },
 { "octpch.i",S(EVAL),   1,      "i",    "i",    octpch                  },
 { "cpspch.i",S(EVAL),   1,      "i",    "i",    cpspch                  },
@@ -418,6 +421,7 @@ OENTRY opcodlst_1[] = {
 { "readk2", S(KREAD2),  3,      "kk",   "Tii",   krd2set, kread2        },
 { "readk3", S(KREAD3),  3,      "kkk",  "Tii",   krd3set, kread3        },
 { "readk4", S(KREAD4),  3,      "kkkk", "Tii",   krd4set, kread4        },
+{ "readks", S(KREADS),  3,      "S",    "Ti",    krdsset, kreads        },
 { "xyin",     S(XYIN),    3,    "kk",   "iiiiioo",xyinset,xyin          },
 { "tempest",  S(TEMPEST), 5,    "k","kiiiiiiiiiop",tempeset,NULL,tempest},
 { "tempo",    S(TEMPO),   3,    "",     "ki",   tempset,tempo           },
@@ -500,8 +504,9 @@ OENTRY opcodlst_1[] = {
 { "filelen", S(SNDINFO), 1,     "i",    "Tp",   filelen, NULL, NULL        },
 { "filenchnls", S(SNDINFO), 1,  "i",    "Tp",   filenchnls, NULL, NULL     },
 { "filesr", S(SNDINFO),  1,     "i",    "Tp",   filesr, NULL, NULL         },
-{ "filebit", S(SNDINFO),  1,     "i",    "Tp",   filebit, NULL, NULL         },
+{ "filebit", S(SNDINFO),  1,     "i",   "Tp",   filebit, NULL, NULL        },
 { "filepeak", S(SNDINFOPEAK), 1, "i",   "To",   filepeak, NULL, NULL       },
+{ "filevalid", S(FILEVALID), 1,  "i",   "S",    filevalid, NULL, NULL      },
 /*  { "nlalp", S(NLALP),     5,     "a",    "akkoo", nlalp_set, NULL, nlalp }, */
 /* Robin Whittle */
 { "tableiw",  S(TABLEW),1,     "",      "iiiooo", (SUBR)itablew, NULL, NULL},
@@ -569,7 +574,9 @@ OENTRY opcodlst_1[] = {
 { "in32",   S(INALL),   4,      "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
                                         "",     NULL,   NULL,   in32 },
 #endif
-{ "inch",   S(INCH),    4,      "a",    "k",    NULL,   NULL,   inch_opcode },
+//{ "inch",   S(INCH),    4,      "a",    "k",    NULL,   NULL,   inch_opcode },
+{ "inch",   S(INCH),    4,      "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+                                        "z",    NULL,   NULL,   inch_opcode },
 #ifndef OLPC
 { "_in",    S(INALL),   4,      "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
                                         "",     NULL,   NULL,   inall_opcode },
