@@ -61,7 +61,8 @@ static void set_xtratim(CSOUND *csound, INSDS *ip)
 {
     if (ip->relesing)
       return;
-    ip->offtim = (csound->icurTime + csound->ksmps * (double) ip->xtratim)/csound->esr;
+    ip->offtim = (csound->icurTime +
+                  csound->ksmps * (double) ip->xtratim)/csound->esr;
     ip->offbet = csound->curBeat + (csound->curBeat_inc * (double) ip->xtratim);
     ip->relesing = 1;
 }
@@ -77,15 +78,31 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
 
     if (csound->advanceCnt)
       return 0;
-    if (UNLIKELY(O->odebug))
-      csound->Message(csound, Str("activating instr %d at %d\n"), insno, csound->icurTime);
+    if (UNLIKELY(O->odebug)) {
+      char *name = csound->instrtxtp[insno]->insname;
+      if (UNLIKELY(name))
+        csound->Message(csound, Str("activating instr %s at %d\n"),
+                        name, csound->icurTime);
+      else
+        csound->Message(csound, Str("activating instr %d at %d\n"),
+                        insno, csound->icurTime);
+    }
     csound->inerrcnt = 0;
     tp = csound->instrtxtp[insno];
     if (UNLIKELY(tp->muted == 0)) {
-      csound->Warning(csound, Str("Instrument %d muted\n"), insno);
+      char *name = csound->instrtxtp[insno]->insname;
+      if (UNLIKELY(name))
+        csound->Warning(csound, Str("Instrument %s muted\n"), name);
+      else
+        csound->Warning(csound, Str("Instrument %d muted\n"), insno);
       return 0;
     }
     if (UNLIKELY(tp->mdepends & 04)) {
+      char *name = csound->instrtxtp[insno]->insname;
+      if (UNLIKELY(name))
+        csound->Message(csound, Str("instr %s expects midi event data, "
+                                    "cannot run from score\n"), name);
+      else
       csound->Message(csound, Str("instr %d expects midi event data, "
                                   "cannot run from score\n"), insno);
       return(1);
@@ -93,14 +110,14 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
     if (tp->cpuload > FL(0.0)) {
       csound->cpu_power_busy += tp->cpuload;
       /* if there is no more cpu processing time*/
-      if (csound->cpu_power_busy > FL(100.0)) {
+      if (UNLIKELY(csound->cpu_power_busy > FL(100.0))) {
         csound->cpu_power_busy -= tp->cpuload;
         csoundWarning(csound, Str("cannot allocate last note because "
                                   "it exceeds 100%% of cpu time"));
         return(0);
       }
     }
-    if (tp->maxalloc > 0 && tp->active >= tp->maxalloc) {
+    if (UNLIKELY(tp->maxalloc > 0 && tp->active >= tp->maxalloc)) {
       csoundWarning(csound, Str("cannot allocate last note because it exceeds "
                                 "instr maxalloc"));
       return(0);
@@ -114,8 +131,13 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
     }
     /* alloc new dspace if needed */
     if (tp->act_instance == NULL) {
-      if (O->msglevel & RNGEMSG)
-        csound->Message(csound, Str("new alloc for instr %d:\n"), insno);
+      if (O->msglevel & RNGEMSG) {
+        char *name = csound->instrtxtp[insno]->insname;
+        if (UNLIKELY(name))
+          csound->Message(csound, Str("new alloc for instr %s:\n"), name);
+        else
+          csound->Message(csound, Str("new alloc for instr %d:\n"), insno);
+      }
       instance(csound, insno);
     }
     /* pop from free instance chain */
@@ -125,6 +147,7 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
 
     /* Add an active instrument */
     tp->active++;
+    tp->instcnt++;
     nxtp = &(csound->actanchor);    /* now splice into activ lst */
     while ((prvp = nxtp) && (nxtp = prvp->nxtact) != NULL) {
       if (nxtp->insno > insno ||
@@ -146,13 +169,19 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
         MYFLT *pfld = &ip->p3;              /* if pset data present */
         MYFLT *pdat = tp->psetdata + 2;
         int32 nn = tp->pmax - 2;             /*   put cur vals in pflds */
-        do {
-          *pfld++ = *pdat++;
-        } while (--nn);
+        /* do { */
+        /*   *pfld++ = *pdat++; */
+        /* } while (--nn); */
+        memcpy(&ip->p3, pdat, sizeof(MYFLT)*nn);
       }
       if ((n = tp->pmax) != newevtp->pcnt && !tp->psetdata) {
-        csoundWarning(csound, Str("instr %d uses %d p-fields but is given %d"),
-                              insno, n, newevtp->pcnt);
+        char *name = csound->instrtxtp[insno]->insname;
+        if (UNLIKELY(name))
+          csoundWarning(csound, Str("instr %s uses %d p-fields but is given %d"),
+                        name, n, newevtp->pcnt);
+        else
+          csoundWarning(csound, Str("instr %d uses %d p-fields but is given %d"),
+                        insno, n, newevtp->pcnt);
       }
       if (newevtp->p3orig >= FL(0.0))
         ip->offbet = csound->beatOffs
@@ -165,7 +194,8 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
         csound->Message(csound, "psave beg at %p\n", (void*) flp);
       if (n > newevtp->pcnt) n = newevtp->pcnt; /* IV - Oct 20 2002 */
       memcpy(flp, fep, n * sizeof(MYFLT)); flp += n;
-      if (n < tp->pmax) memset(flp, 0, (tp->pmax - n) * sizeof(MYFLT));
+      if (n < tp->pmax && tp->psetdata==NULL)
+        memset(flp, 0, (tp->pmax - n) * sizeof(MYFLT));
       if (UNLIKELY(O->odebug))
         csound->Message(csound, "   ending at %p\n", (void*) flp);
     }
@@ -220,7 +250,11 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
       ip->offtim = -1.0;                        /*   else mark indef     */
     }
     if (UNLIKELY(O->odebug)) {
-      csound->Message(csound, Str("instr %d now active:\n"), insno);
+      char *name = csound->instrtxtp[insno]->insname;
+      if (UNLIKELY(name))
+        csound->Message(csound, Str("instr %s now active:\n"), name);
+      else
+        csound->Message(csound, Str("instr %d now active:\n"), insno);
       showallocs(csound);
     }
     return 0;
@@ -243,7 +277,7 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
     tp = csound->instrtxtp[insno];
     if (tp->cpuload > FL(0.0)) {
       csound->cpu_power_busy += tp->cpuload;
-      if (csound->cpu_power_busy > FL(100.0)) {
+      if (UNLIKELY(csound->cpu_power_busy > FL(100.0))) {
         /* if there is no more cpu time */
         csound->cpu_power_busy -= tp->cpuload;
         csoundWarning(csound, Str("cannot allocate last note because "
@@ -257,14 +291,25 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
       return(0);
     }
     tp->active++;
-    if (UNLIKELY(O->odebug))
-      csound->Message(csound, Str("activating instr %d\n"), insno);
+    tp->instcnt++;
+    if (UNLIKELY(O->odebug)) {
+      char *name = csound->instrtxtp[insno]->insname;
+      if (UNLIKELY(name))
+        csound->Message(csound, Str("activating instr %s\n"), name);
+      else
+        csound->Message(csound, Str("activating instr %d\n"), insno);
+    }
     csound->inerrcnt = 0;
     ipp = &chn->kinsptr[mep->dat1];       /* key insptr ptr           */
     /* alloc new dspace if needed */
     if (tp->act_instance == NULL) {
-      if (O->msglevel & RNGEMSG)
-        csound->Message(csound, Str("new alloc for instr %d:\n"), insno);
+      if (O->msglevel & RNGEMSG) {
+        char *name = csound->instrtxtp[insno]->insname;
+        if (UNLIKELY(name))
+          csound->Message(csound, Str("new alloc for instr %s:\n"), name);
+        else
+          csound->Message(csound, Str("new alloc for instr %d:\n"), insno);
+      }
       instance(csound, insno);
     }
     /* pop from free instance chain */
@@ -274,7 +319,7 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
 
     if (UNLIKELY(O->odebug))
       csound->Message(csound, "Now %d active instr %d\n", tp->active, insno);
-    if ((prvp = *ipp) != NULL) {          /*   if key currently activ */
+    if (UNLIKELY((prvp = *ipp) != NULL)) {          /*   if key currently activ */
       csoundWarning(csound,
                     Str("MIDI note overlaps with key %d on same channel"),
                     (int) mep->dat1);
@@ -343,7 +388,7 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
       value = value * OCTRES;
       value = (MYFLT) CPSOCTL((int32) value);
       pfields[index] = value;
-      if (O->msglevel & WARNMSG) {
+      if (UNLIKELY(O->msglevel & WARNMSG)) {
         csound->Message(csound, "  midiKeyCps:      pfield: %3d  value: %3d\n",
                         pfield, (int) pfields[index]);
       }
@@ -383,7 +428,7 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
       MYFLT *pfields = &ip->p1;
       MYFLT value = (MYFLT) ip->m_veloc;
       pfields[index] = value;
-      if (O->msglevel & WARNMSG) {
+      if (UNLIKELY(O->msglevel & WARNMSG)) {
         csound->Message(csound, "  midiVelocity:    pfield: %3d  value: %3d\n",
                         pfield, (int) pfields[index]);
       }
@@ -396,7 +441,7 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
       value = value * value / FL(16239.0);
       value = value * csound->e0dbfs;
       pfields[index] = value;
-      if (O->msglevel & WARNMSG) {
+      if (UNLIKELY(O->msglevel & WARNMSG)) {
         csound->Message(csound, "  midiVelocityAmp: pfield: %3d  value: %3d\n",
                         pfield, (int) pfields[index]);
       }
@@ -417,7 +462,11 @@ int MIDIinsert(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
       return csound->inerrcnt;
     }
     if (UNLIKELY(O->odebug)) {
-      csound->Message(csound, Str("instr %d now active:\n"), insno);
+      char *name = csound->instrtxtp[insno]->insname;
+      if (UNLIKELY(name))
+        csound->Message(csound, Str("instr %s now active:\n"), name);
+      else
+        csound->Message(csound, Str("instr %d now active:\n"), insno);
       showallocs(csound);
     }
     return 0;
@@ -515,8 +564,13 @@ static void deact(CSOUND *csound, INSDS *ip)
       ((SUBINST*) ip->subins_deact)->ip = NULL;
       ip->subins_deact = NULL;
     }
-    if (UNLIKELY(csound->oparms->odebug))
-      csound->Message(csound, Str("removed instance of instr %d\n"), ip->insno);
+    if (UNLIKELY(csound->oparms->odebug)) {
+      char *name = csound->instrtxtp[ip->insno]->insname;
+      if (UNLIKELY(name))
+        csound->Message(csound, Str("removed instance of instr %s\n"), name);
+      else
+        csound->Message(csound, Str("removed instance of instr %d\n"), ip->insno);
+    }
     /* IV - Oct 24 2002: ip->prvact may be NULL, so need to check */
     if (ip->prvact && (nxtp = ip->prvact->nxtact = ip->nxtact) != NULL)
       nxtp->prvact = ip->prvact;
@@ -651,7 +705,8 @@ void infoff(CSOUND *csound, MYFLT p1)   /* turn off an indef copy of instr p1 */
       do {
         if (ip->insno == insno          /* if find the insno */
             && ip->actflg               /*      active       */
-           /* && ip->offtim < 0.0 */         /*      but indef,   */
+            && ip->offtim < 0.0          /*  but indef, VL: currently this condition
+            cannot be removed, as it breaks turning off extratime instances */
             && ip->p1 == p1) {
           if (UNLIKELY(csound->oparms->odebug))
             csound->Message(csound, "turning off inf copy of instr %d\n",
@@ -687,12 +742,12 @@ int csoundInitError(CSOUND *csound, const char *s, ...)
       do {
         ip = ((OPCOD_IOBUFS*) ip->opcod_iobufs)->parent_ip;
       } while (ip->opcod_iobufs);
-      if (op)
+      if (op) 
         sprintf(buf, Str("INIT ERROR in instr %d (opcode %s): "),
-                     ip->insno, op->name);
+                ip->insno, op->name);
       else
         sprintf(buf, Str("INIT ERROR in instr %d (subinstr %d): "),
-                     ip->insno, csound->ids->insdshead->insno);
+                ip->insno, csound->ids->insdshead->insno);
     }
     else
       sprintf(buf, Str("INIT ERROR in instr %d: "), ip->insno);
@@ -779,6 +834,7 @@ int subinstrset(CSOUND *csound, SUBINST *p)
       p->ip->insno = (int16) instno;
       p->ip->actflg++;                  /*    and mark the instr active */
       csound->instrtxtp[instno]->active++;
+      csound->instrtxtp[instno]->instcnt++;
       p->ip->p1 = (MYFLT) instno;
       p->ip->opcod_iobufs = (void*) &p->buf;
       /* link into deact chain */
@@ -898,6 +954,7 @@ int useropcdset(CSOUND *csound, UOPCODE *p)
       tp->act_instance = lcurip->nxtact;    /* remove from chain      */
       lcurip->actflg++;                     /*    and mark the instr active */
       tp->active++;
+      tp->instcnt++;
       /* link into deact chain */
       lcurip->opcod_deact = parent_ip->opcod_deact;
       lcurip->subins_deact = NULL;
@@ -1177,8 +1234,13 @@ INSDS *insert_event(CSOUND *csound,
     if (UNLIKELY(O->odebug))
       csound->Message(csound, "activating instr %d\n", insno);
     if (UNLIKELY((tp->mdepends & 4) && !midi)) {
-      csound->Message(csound, Str("instr %d expects midi event data, "
-                                  "cannot run from score\n"), insno);
+      char *name = csound->instrtxtp[ip->insno]->insname;
+      if (name)
+        csound->Message(csound, Str("instr %s expects midi event data, "
+                                    "cannot run from score\n"), name);
+      else 
+        csound->Message(csound, Str("instr %d expects midi event data, "
+                                    "cannot run from score\n"), insno);
       csound->perferrcnt++;
       goto endsched;
     }
@@ -1192,8 +1254,13 @@ INSDS *insert_event(CSOUND *csound,
     }
     /* alloc new dspace if needed */
     if (tp->act_instance == NULL) {
-      if (O->msglevel & RNGEMSG)
+      if (O->msglevel & RNGEMSG) {
+      char *name = csound->instrtxtp[insno]->insname;
+      if (name)
+        csound->Message(csound, Str("new alloc for instr %s:\n"), name);
+      else
         csound->Message(csound, Str("new alloc for instr %d:\n"), insno);
+      }
       instance(csound, insno);
     }
     /* pop from free instance chain */
@@ -1203,6 +1270,7 @@ INSDS *insert_event(CSOUND *csound,
 
     /* Add an active instrument */
     tp->active++;
+    tp->instcnt++;
     nxtp = &(csound->actanchor);    /* now splice into active list */
     while ((prvp = nxtp) && (nxtp = prvp->nxtact) != NULL)
       if (nxtp->insno > insno    ||
@@ -1220,8 +1288,13 @@ INSDS *insert_event(CSOUND *csound,
       int imax = tp->pmax - 3;
       MYFLT  *flp;
       if (UNLIKELY((int) tp->pmax != pcnt)) {
-        csoundWarning(csound, Str("instr %d pmax = %d, note pcnt = %d"),
-                              insno, (int) tp->pmax, pcnt);
+        char *name = csound->instrtxtp[insno]->insname;
+        if (name)
+          csoundWarning(csound, Str("instr %s pmax = %d, note pcnt = %d"),
+                      name, (int) tp->pmax, pcnt);
+        else
+          csoundWarning(csound, Str("instr %d pmax = %d, note pcnt = %d"),
+                        insno, (int) tp->pmax, pcnt);
       }
       ip->p1 = instr;
       ip->p2 = when;
@@ -1789,8 +1862,15 @@ int delete_instr(CSOUND *csound, DELETEIN *p)
     active = ip->instance;
     while (active != NULL) {    /* Check there are no active instances */
       INSDS   *nxt = active->nxtinstance;
-      if (UNLIKELY(active->actflg))  /* Can only remove non-active instruments */
-        return csound->InitError(csound, Str("Instrument %d is still active"), n);
+      if (UNLIKELY(active->actflg)) { /* Can only remove non-active instruments */
+        char *name = csound->instrtxtp[n]->insname;
+        if (name)
+          return csound->InitError(csound,
+                                   Str("Instrument %s is still active"), name);
+        else
+          return csound->InitError(csound,
+                                   Str("Instrument %d is still active"), n);
+      }
 #if 0
       if (active->opcod_iobufs && active->insno > csound->maxinsno)
         mfree(csound, active->opcod_iobufs);            /* IV - Nov 10 2002 */
