@@ -262,7 +262,7 @@ int musmon(CSOUND *csound)
     }
     csound->Message(csound, Str("audio buffered in %d sample-frame blocks\n"),
                             (int) O->outbufsamps);
-    O->inbufsamps *= csound->nchnls_i;    /* now adjusted for n channels  */
+    O->inbufsamps *= csound->inchnls;    /* now adjusted for n channels  */
     O->outbufsamps *= csound->nchnls;
     iotranset(csound);          /* point recv & tran to audio formatter */
       /* open audio file or device for input first, and then for output */
@@ -307,7 +307,7 @@ int musmon(CSOUND *csound)
       if (UNLIKELY(!(csound->scfp = fopen("cscore.out", "r"))))
         csoundDie(csound, Str("cannot reopen cscore.out"));
       csoundNotifyFileOpened(csound, "cscore.out", CSFTYPE_SCORE_OUT, 0, 0);
-      /* write to cscore.srt */ 
+      /* write to cscore.srt */
      if (UNLIKELY(!(csound->oscfp = fopen("cscore.srt", "w"))))
         csoundDie(csound, Str("cannot reopen cscore.srt"));
       csoundNotifyFileOpened(csound, "cscore.srt", CSFTYPE_SCORE_OUT, 1, 0);
@@ -728,7 +728,13 @@ static void process_midi_event(CSOUND *csound, MEVENT *mep, MCHNBLK *chn)
       if (UNLIKELY((n = MIDIinsert(csound, insno, chn, mep)))) {     /* alloc,init,activ */
         csound->Message(csound,
                         Str("\t\t   T%7.3f - note deleted. "), csound->curp2);
-        csound->Message(csound, Str("instr %d had %d init errors\n"), insno, n);
+        {
+          char *name = csound->instrtxtp[insno]->insname;
+          if (name)
+            csound->Message(csound, Str("instr %s had %d init errors\n"), name, n);
+          else
+            csound->Message(csound, Str("instr %d had %d init errors\n"), insno, n);
+        }
         csound->perferrcnt++;
       }
     }
