@@ -185,10 +185,15 @@ void init_symbtab(CSOUND *csound)
     add_token(csound, "tb15", T_FUNCTION);
     add_token(csound, "urd", T_FUNCTION);
     add_token(csound, "not", T_FUNCTION);
-
+    add_token(csound, "cent", T_FUNCTION);
+    add_token(csound, "octave", T_FUNCTION);
+    add_token(csound, "semitone", T_FUNCTION);
+    add_token(csound, "cpsmidinn", T_FUNCTION);
+    add_token(csound, "octmidinn", T_FUNCTION);
+    add_token(csound, "pchmidinn", T_FUNCTION);
 }
 
-unsigned int hash(char *s)
+static unsigned int hash(char *s)
 {
     unsigned int h = 0;
     while (*s != '\0') {
@@ -199,7 +204,7 @@ unsigned int hash(char *s)
 
 ORCTOKEN *add_token(CSOUND *csound, char *s, int type)
 {
-    int h = hash(s);
+    unsigned int h = hash(s);
 
     //printf("Hash value for %s: %i\n", s, h);
 
@@ -208,8 +213,12 @@ ORCTOKEN *add_token(CSOUND *csound, char *s, int type)
     while (a!=NULL) {
       if (strcmp(a->lexeme, s)==0) {
         if (type == a->type) return a;
-        //printf("Type confusion for %s, stopping\n", s);
-        exit(1);
+        if (type!= T_FUNCTION || a->type!=T_OPCODE) 
+          csound->Warning(csound,
+                          Str("Type confusion for %s (%d,%d), replacing\n"),
+                          s, type, a->type);
+        a->type = type;
+        return a;
       }
       a = a->next;
     }
@@ -228,7 +237,7 @@ int isUDOArgList(char *s)
     int len = strlen(s) - 1;
 
     while (len >= 0) {
-      if (UNLIKELY(strchr("aijkKop0", s[len]) == NULL)) {
+      if (UNLIKELY(strchr("aijkKopS0", s[len]) == NULL)) {
         /* printf("Invalid char '%c' in '%s'", *p, s); */
         return 0;
       }
@@ -242,7 +251,7 @@ int isUDOAnsList(char *s)
     int len = strlen(s) - 1;
 
     while (len >= 0) {
-      if (UNLIKELY(strchr("aikK0", s[len]) == NULL)) {
+      if (UNLIKELY(strchr("aikSK0", s[len]) == NULL)) {
         return 0;
       }
       len--;
@@ -252,7 +261,7 @@ int isUDOAnsList(char *s)
 
 ORCTOKEN *lookup_token(CSOUND *csound, char *s)
 {
-    int h = hash(s);
+    unsigned int h = hash(s);
     int type = T_IDENT;
     ORCTOKEN *a = symbtab[h];
     ORCTOKEN *ans;
