@@ -265,3 +265,18 @@ int filepeak(CSOUND *csound, SNDINFOPEAK *p)
     return OK;
 }
 
+/* From matt ingalls */
+int filevalid(CSOUND *csound, FILEVALID *p)
+{
+    char soundiname[512];
+    *p->r1 = 0;
+    csound->strarg2name(csound, soundiname, p->ifilno, "soundin.",
+                        p->XSTRCODE);
+    if (UNLIKELY(strcmp(soundiname, "-i") == 0)) {    /* get info on the -i    */
+      if (csound->oparms->infilename)  /* commandline inputfile */
+        *p->r1 = 1;
+    }
+    if (LIKELY(csound->FindInputFile(csound, soundiname, "SFDIR;SSDIR")))
+      *p->r1 = 1;
+    return OK;
+}
