@@ -35,22 +35,23 @@ static int krsnsetx(CSOUND *csound, KRESONX *p)
 {
     int scale;
     p->scale = scale = (int) *p->iscl;
-    if ((p->loop = (int) (*p->ord + FL(0.5))) < 1)
+    if ((p->loop = MYFLT2LRND(*p->ord)) < 1)
       p->loop = 4; /*default value*/
     if (!*p->istor && (p->aux.auxp == NULL ||
-                      (int)(p->loop*2*sizeof(MYFLT)) > p->aux.size))
+                       (int)(p->loop*2*sizeof(MYFLT)) > p->aux.size))
       csound->AuxAlloc(csound, (long)(p->loop*2*sizeof(MYFLT)), &p->aux);
     p->yt1 = (MYFLT*)p->aux.auxp; p->yt2 = (MYFLT*)p->aux.auxp + p->loop;
     if (scale && scale != 1 && scale != 2) {
       return csound->InitError(csound,Str("illegal reson iscl value, %f"),
                                *p->iscl);
     }
-    p->prvcf = p->prvbw = -FL(100.0);
-
     if (!(*p->istor)) {
-      int j;
-      for (j=0; j< p->loop; j++) p->yt1[j] = p->yt2[j] = FL(0.0);
+      memset(p->yt1, 0, p->loop*sizeof(MYFLT));
+      memset(p->yt2, 0, p->loop*sizeof(MYFLT));
+      /* int j; */
+      /* for (j=0; j< p->loop; j++) p->yt1[j] = p->yt2[j] = FL(0.0); */
     }
+    p->prvcf = p->prvbw = -FL(100.0);
     return OK;
 }
 
@@ -63,12 +64,12 @@ static int kresonx(CSOUND *csound, KRESONX *p) /* Gabriel Maldonado, modified */
 
     if (*p->kcf != p->prvcf) {
       p->prvcf = *p->kcf;
-      p->cosf = (MYFLT) cos((double)(*p->kcf * csound->tpidsr * csound->ksmps));
+      p->cosf = COS(*p->kcf * csound->tpidsr * csound->ksmps);
       flag = 1;
     }
     if (*p->kbw != p->prvbw) {
       p->prvbw = *p->kbw;
-      p->c3 = (MYFLT) exp((double)(*p->kbw * csound->mtpdsr * csound->ksmps));
+      p->c3 = EXP(*p->kbw * csound->mtpdsr * csound->ksmps);
       flag = 1;
     }
     if (flag) {
@@ -78,9 +79,9 @@ static int kresonx(CSOUND *csound, KRESONX *p) /* Gabriel Maldonado, modified */
       p->c2 = c3t4 * p->cosf / c3p1;            /* -B, so + below */
       c2sqr = p->c2 * p->c2;
       if (p->scale == 1)
-        p->c1 = omc3 * (MYFLT)sqrt(1.0 - (double)(c2sqr / c3t4));
+        p->c1 = omc3 * SQRT(FL(1.0) - (c2sqr / c3t4));
       else if (p->scale == 2)
-        p->c1 = (MYFLT)sqrt((double)((c3p1*c3p1-c2sqr) * omc3/c3p1));
+        p->c1 = SQRT((c3p1*c3p1-c2sqr) * omc3/c3p1);
       else p->c1 = FL(1.0);
     }
     c1   = p->c1;
@@ -91,11 +92,9 @@ static int kresonx(CSOUND *csound, KRESONX *p) /* Gabriel Maldonado, modified */
     asig = p->asig;
     ar   = p->ar;
     for (j=0; j< p->loop; j++) {
-      *ar = c1 * *asig + c2 * *yt1 - c3 * *yt2;
-      *yt2 = *yt1;
-      *yt1 = *ar;
-      yt1++;
-      yt2++;
+      *ar = c1 * *asig + c2 * yt1[j] - c3 * yt2[j];
+      yt2[j] = yt1[j];
+      yt1[j] = *ar;
       asig= p->ar;
     }
     return OK;
@@ -110,6 +109,7 @@ static int fastab_set(CSOUND *csound, FASTAB *p)
       return csound->InitError(csound, Str("fastab: incorrect table number"));
     }
     p->table = ftp->ftable;
+    p->tablen = ftp->flen;
     p->xmode = (int) *p->ixmode;
     if (p->xmode)
       p->xbmul = (MYFLT) ftp->flen;
@@ -125,66 +125,89 @@ static int fastabw(CSOUND *csound, FASTAB *p)
     MYFLT *rslt = p->rslt, *ndx = p->xndx;
     if (p->xmode) {
       MYFLT xbmul = p->xbmul;   /* load once */
-      for (n=0; n<nsmps; n++)   /* for loops compile better */
-        tab[(int)(ndx[n]*xbmul)] = rslt[n];
+      for (n=0; n<nsmps; n++)  { /* for loops compile better */
+        int i = (int)(ndx[n]*xbmul);
+        if (UNLIKELY(i >= p->tablen || i<0)) {
+          return csound->PerfError(csound, Str("tabw off end"));
+        }
+        tab[i] = rslt[n];
+      }
     }
     else {
-      for (n=0; n<nsmps; n++)
-        tab[(int)ndx[n]] = rslt[n];
+      for (n=0; n<nsmps; n++) {
+        int i = ndx[n];
+        if (UNLIKELY(i >= p->tablen || i<0)) {
+          return csound->PerfError(csound, Str("tabw off end"));
+        }
+        tab[i] = rslt[n];
+      }
     }
     return OK;
 }
 
 static int fastabk(CSOUND *csound, FASTAB *p)
 {
+    int i;
     if (p->xmode)
-      *p->rslt =  *(p->table + (int) (*p->xndx * p->xbmul));
+      i = (int) (*p->xndx * p->xbmul);
     else
-      *p->rslt =  *(p->table + (int) *p->xndx);
+      i = (int) *p->xndx;
+    if (UNLIKELY(i >= p->tablen || i<0)) {
+      return csound->PerfError(csound, Str("tab off end"));
+    }
+    *p->rslt =  p->table[i];
     return OK;
 }
 
 static int fastabkw(CSOUND *csound, FASTAB *p)
 {
+    int i;
     if (p->xmode)
-      *(p->table + (int) (*p->xndx * p->xbmul)) = *p->rslt;
+      i = (int) (*p->xndx * p->xbmul);
     else
-      *(p->table + (int) *p->xndx) = *p->rslt;
+      i = (int) *p->xndx;
+    if (UNLIKELY(i >= p->tablen || i<0)) {
+      return csound->PerfError(csound, Str("tabw off end"));
+    }
+    p->table[i] = *p->rslt;
     return OK;
 }
 
 static int fastabi(CSOUND *csound, FASTAB *p)
 {
     FUNC *ftp;
-    /*ftp = csound->FTFind(p->xfn); */
+    int i;
 
     if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL)) {
       return csound->InitError(csound, Str("tab_i: incorrect table number"));
     }
     if (*p->ixmode)
-      *p->rslt =  *(ftp->ftable + (int) (*p->xndx * ftp->flen));
-    else {
-      if ((int) *p->xndx >= ftp->flen) {
-        printf("Table %d: index=%d length=%d\n",
-               ftp->fno, (int) *p->xndx, ftp->flen);
-        return csound->PerfError(csound, "tab_i off end");
-      }
-      *p->rslt =  *(ftp->ftable + (int) *p->xndx);
+      i = (int) (*p->xndx * ftp->flen);
+    else
+      i = (int) *p->xndx;
+    if (UNLIKELY(i >= ftp->flen || i<0)) {
+        return csound->PerfError(csound, Str("tab_i off end"));
     }
+    *p->rslt =  ftp->ftable[i];
     return OK;
 }
 
 static int fastabiw(CSOUND *csound, FASTAB *p)
 {
     FUNC *ftp;
+    int i;
     /*ftp = csound->FTFind(p->xfn); */
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
       return csound->InitError(csound, Str("tabw_i: incorrect table number"));
     }
     if (*p->ixmode)
-      *(ftp->ftable + (int) (*p->xndx * ftp->flen)) = *p->rslt;
+      i = (int) (*p->xndx * ftp->flen);
     else
-      *(ftp->ftable + (int) *p->xndx) = *p->rslt;
+      i = (int) *p->xndx;
+    if (UNLIKELY(i >= ftp->flen || i<0)) {
+        return csound->PerfError(csound, Str("tabw_i off end"));
+    }
+    ftp->ftable[i] = *p->rslt;
     return OK;
 }
 
@@ -196,12 +219,20 @@ static int fastab(CSOUND *csound, FASTAB *p)
     if (p->xmode) {
       MYFLT xbmul = p->xbmul;
       for (i=0; i<nsmps; i++) {
-        rslt[i] = tab[(int) (ndx[i] * xbmul)];
+        int n = (int) (ndx[i] * xbmul);
+        if (UNLIKELY(n >= p->tablen || n<0)) {
+          return csound->PerfError(csound, Str("tab off end"));
+        }
+        rslt[i] = tab[n];
       }
     }
     else {
       for (i=0; i<nsmps; i++) {
-        rslt[i] = tab[(int) ndx[i]];
+        int n = (int) ndx[i];
+        if (UNLIKELY(n >= p->tablen || n<0)) {
+          return csound->PerfError(csound, Str("tab off end"));
+        }
+        rslt[i] = tab[n];
       }
     }
     return OK;
@@ -211,7 +242,7 @@ static CS_NOINLINE int tab_init(CSOUND *csound, TB_INIT *p, int ndx)
 {
     MYFLT             *ft;
     STDOPCOD_GLOBALS  *pp;
-    if (UNLIKELY(csound->GetTable(csound, &ft, (int) *(p->ifn)) < 0))
+    if (UNLIKELY(csound->GetTable(csound, &ft, MYFLT2LRND(*p->ifn)) < 0))
       return csound->InitError(csound, Str("tab_init: incorrect table number"));
     pp = (STDOPCOD_GLOBALS*) csound->stdOp_Env;
     pp->tb_ptrs[ndx] = ft;
@@ -301,7 +332,7 @@ static void printi(CSOUND *csound, PRINTI *p)
 /* opcodes from Jens Groh */
 /* ====================== */
 
-static int nlalp_set(CSOUND *csound,NLALP *p)
+static int nlalp_set(CSOUND *csound, NLALP *p)
 {
    if (!(*p->istor)) {
      p->m0 = 0.0;
@@ -310,66 +341,66 @@ static int nlalp_set(CSOUND *csound,NLALP *p)
    return OK;
 }
 
-static int nlalp(CSOUND *csound,NLALP *p)
+static int nlalp(CSOUND *csound, NLALP *p)
 {
-   int nsmps;
-   MYFLT *rp;
-   MYFLT *ip;
-   double m0;
-   double m1;
-   double tm0;
-   double tm1;
-   double klfact;
-   double knfact;
+    int nsmps = csound->ksmps, n;
+    MYFLT *rp;
+    MYFLT *ip;
+    double m0;
+    double m1;
+    double tm0;
+    double tm1;
+    double klfact;
+    double knfact;
 
-   nsmps = csound->ksmps;
-   rp = p->aresult;
-   ip = p->ainsig;
-   klfact = (double)*p->klfact;
-   knfact = (double)*p->knfact;
-   tm0 = p->m0;
-   tm1 = p->m1;
-   if (knfact == 0.0) { /* linear case */
-     if (klfact == 0.0) { /* degenerated linear case */
-       m0 = (double)*ip++ - tm1;
-       *rp++ = (MYFLT)(tm0);
-       while (--nsmps) {
-         *rp++ = (MYFLT)(m0);
-         m0 = (double)*ip++;
-       }
-       tm0 = m0;
-       tm1 = 0.0;
-     } else { /* normal linear case */
-       do {
-         m0 = (double)*ip++ - tm1;
-         m1 = m0 * klfact;
-         *rp++ = (MYFLT)(tm0 + m1);
-         tm0 = m0;
-         tm1 = m1;
-       } while (--nsmps);
-     }
-   } else { /* non-linear case */
-     if (klfact == 0.0) { /* simplified non-linear case */
-       do {
-         m0 = (double)*ip++ - tm1;
-         m1 = fabs(m0) * knfact;
-         *rp++ = (MYFLT)(tm0 + m1);
-         tm0 = m0;
-         tm1 = m1;
-       } while (--nsmps);
-     } else { /* normal non-linear case */
-       do {
-         m0 = (double)*ip++ - tm1;
-         m1 = m0 * klfact + fabs(m0) * knfact;
-         *rp++ = (MYFLT)(tm0 + m1);
-         tm0 = m0;
-         tm1 = m1;
-       } while (--nsmps);
-     }
-   }
-   p->m0 = tm0;
-   p->m1 = tm1;
-   return OK;
+    rp = p->aresult;
+    ip = p->ainsig;
+    klfact = (double)*p->klfact;
+    knfact = (double)*p->knfact;
+    tm0 = p->m0;
+    tm1 = p->m1;
+    if (knfact == 0.0) { /* linear case */
+      if (klfact == 0.0) { /* degenerated linear case */
+        m0 = (double)ip[0] - tm1;
+        rp[0] = (MYFLT)(tm0);
+        for (n=1; n<nsmps; n++) {
+          rp[n] = (MYFLT)(m0);
+          m0 = (double)ip[n];
+        }
+        tm1 = 0.0;
+        tm0 = m0;
+      }
+      else { /* normal linear case */
+        for (n=0; n<nsmps; n++) {
+          m0 = (double)ip[n] - tm1;
+          m1 = m0 * klfact;
+          rp[n] = (MYFLT)(tm0 + m1);
+          tm1 = m1;
+          tm0 = m0;
+        }
+      }
+    } else { /* non-linear case */
+      if (klfact == 0.0) { /* simplified non-linear case */
+        for (n=0; n<nsmps; n++) {
+          m0 = (double)ip[n] - tm1;
+          m1 = fabs(m0) * knfact;
+          rp[n] = (MYFLT)(tm0 + m1);
+          tm1 = m1;
+          tm0 = m0;
+        }
+      } else { /* normal non-linear case */
+        for (n=0; n<nsmps; n++) {
+          m0 = (double)ip[n] - tm1;
+          m1 = m0 * klfact + fabs(m0) * knfact;
+          rp[n] = (MYFLT)(tm0 + m1);
+          tm1 = m1;
+          tm0 = m0;
+        }
+      }
+    }
+    p->m0 = tm0;
+    p->m1 = tm1;
+    return OK;
 }
 
 /* ----------------------------------------------- */
@@ -377,13 +408,13 @@ static int nlalp(CSOUND *csound,NLALP *p)
 static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
 {
     FUNC    *ftp;
-    MYFLT fmaxlen = (MYFLT)MAXLEN;
     int     count;
-    long    *lphs;
-    MYFLT       *pAmp;
+    int32   *lphs;
+    MYFLT   iphs = *p->iphs;
+
     p->inerr = 0;
 
-    if ((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
     }
     else {
@@ -392,32 +423,31 @@ static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
     }
 
     count = (int)*p->icnt;
-    if (count < 1)
-      count = 1;
+    if (UNLIKELY(count < 1)) count = 1;
     p->count = count;
 
-    if ((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifreqtbl)) != NULL)) {
       p->freqtp = ftp;
     }
     else {
       p->inerr = 1;
       return csound->InitError(csound, Str("adsynt2: freqtable not found!"));
     }
-    if (ftp->flen < count) {
+    if (UNLIKELY(ftp->flen < count)) {
       p->inerr = 1;
       return csound->InitError(csound,
                                Str("adsynt2: partial count is greater "
                                    "than freqtable size!"));
     }
 
-    if ((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->iamptbl)) != NULL)) {
       p->amptp = ftp;
     }
     else {
       p->inerr = 1;
       return csound->InitError(csound, Str("adsynt2: amptable not found!"));
     }
-    if (ftp->flen < count) {
+    if (UNLIKELY(ftp->flen < count)) {
       p->inerr = 1;
       return csound->InitError(csound,
                                Str("adsynt2: partial count is greater "
@@ -425,41 +455,47 @@ static int adsynt2_set(CSOUND *csound,ADSYNT2 *p)
     }
 
     if (p->lphs.auxp==NULL ||
-        p->lphs.size < (long)(sizeof(long)+sizeof(MYFLT))*count)
-      csound->AuxAlloc(csound, (sizeof(long)+sizeof(MYFLT))*count, &p->lphs);
+        p->lphs.size < (int32)(sizeof(int32)*count))
+      csound->AuxAlloc(csound, sizeof(int32)*count, &p->lphs);
+    lphs = (int32*)p->lphs.auxp;
 
-    lphs = (long*)p->lphs.auxp;
-    if (*p->iphs > 1) {
-      do {
-        *lphs++ = ((long)
+    if (iphs > 1) {
+      int c;
+      for (c=0; c<count; c++) {
+        lphs[c] = ((int32)
                    ((MYFLT) ((double) (csound->Rand31(&(csound->randSeed1)) - 1)
-                             / 2147483645.0) * fmaxlen)) & PHMASK;
-      } while (--count);
+                             / 2147483645.0) * FMAXLEN)) & PHMASK;
+      }
     }
-    else if (*p->iphs >= 0) {
-      do {
-        *lphs++ = ((long)(*p->iphs * fmaxlen)) & PHMASK;
-      } while (--count);
+    else if (iphs >= 0) {
+      int c;
+      for (c=0; c<count; c++) {
+        lphs[c] = ((int32)(iphs * FMAXLEN)) & PHMASK;
+      }
     }
-    pAmp = p->previousAmp = (MYFLT *) lphs + sizeof(MYFLT)*count;
-    count = (int)*p->icnt;
-    do {
-      *pAmp++ = FL(0.0);
-    } while (--count);
+    if (p->pamp.auxp==NULL ||
+        p->pamp.size < (int32)(sizeof(MYFLT)*p->count))
+      csound->AuxAlloc(csound, sizeof(MYFLT)*p->count, &p->pamp);
+    else                        /* AuxAlloc clear anyway */
+      memset(p->pamp.auxp, 0, sizeof(MYFLT)*p->count);
+    /* count = (int)*p->icnt; */
+    /* do { */
+    /*   *pAmp++ = FL(0.0); */
+    /* } while (--count); */
     return OK;
 }
 
 static int adsynt2(CSOUND *csound,ADSYNT2 *p)
 {
     FUNC    *ftp, *freqtp, *amptp;
-    MYFLT   *ar, *ar0, *ftbl, *freqtbl, *amptbl, *prevAmp;
-    MYFLT   amp0, amp, cps0, cps, ampIncr,amp2;
-    long    phs, inc, lobits;
-    long    *lphs;
-    int     nsmps, count;
+    MYFLT   *ar, *ftbl, *freqtbl, *amptbl, *prevAmp;
+    MYFLT   amp0, amp, cps0, cps, ampIncr, amp2;
+    int32   phs, inc, lobits;
+    int32   *lphs;
+    int     c, n, nsmps= csound->ksmps, count;
 
-    if (p->inerr) {
-      return csound->InitError(csound, Str("adsynt2: not initialized"));
+    if (UNLIKELY(p->inerr)) {
+      return csound->InitError(csound, Str("adsynt2: not initialised"));
     }
     ftp = p->ftp;
     ftbl = ftp->ftable;
@@ -468,40 +504,32 @@ static int adsynt2(CSOUND *csound,ADSYNT2 *p)
     freqtbl = freqtp->ftable;
     amptp = p->amptp;
     amptbl = amptp->ftable;
-    lphs = (long*)p->lphs.auxp;
-    prevAmp = p->previousAmp;
+    lphs = (int32*)p->lphs.auxp;
+    prevAmp = (MYFLT*)p->pamp.auxp;
 
     cps0 = *p->kcps;
     amp0 = *p->kamp;
     count = p->count;
 
-    ar0 = p->sr;
-    ar = ar0;
-    nsmps = csound->ksmps;
-    do {
-      *ar++ = FL(0.0);
-    } while (--nsmps);
+    ar = p->sr;
+    memset(ar, 0, nsmps*sizeof(MYFLT));
 
-    do {
-      ar = ar0;
-      nsmps = csound->ksmps;
-      amp2 = *prevAmp;
-      amp = *amptbl++ * amp0;
-      cps = *freqtbl++ * cps0;
-      inc = (long) (cps * csound->sicvt);
-      phs = *lphs;
-      ampIncr = (amp - *prevAmp) * csound->onedksmps;
-      do {
-        *ar++ += *(ftbl + (phs >> lobits)) * amp2;
+    for (c=0; c<count; c++) {
+      amp2 = prevAmp[c];
+      amp = amptbl[c] * amp0;
+      cps = freqtbl[c] * cps0;
+      inc = (int32) (cps * csound->sicvt);
+      phs = lphs[c];
+      ampIncr = (amp - amp2) * csound->onedksmps;
+      for (n=0; n<nsmps; n++) {
+        ar[n] += *(ftbl + (phs >> lobits)) * amp2;
         phs += inc;
         phs &= PHMASK;
         amp2 += ampIncr;
       }
-      while (--nsmps);
-      *prevAmp++ = amp;
-      *lphs++ = phs;
+      prevAmp[c] = amp;
+      lphs[c] = phs;
     }
-    while (--count);
     return OK;
 }
 
@@ -635,9 +663,11 @@ static int tabplay_k(CSOUND *csound,TABPLAY *p)
     return OK;
 }
 
+/* This code is wrong; old_inargs is not initialised */
 static int isChanged_set(CSOUND *csound,ISCHANGED *p)
 {
     p->numargs = p->INOCOUNT;
+    memset(p->old_inargs, 0, sizeof(MYFLT)*p->numargs); /* Initialise */
     return OK;
 }
 
@@ -674,37 +704,35 @@ static int partial_maximum_set(CSOUND *csound,P_MAXIMUM *p)
 
 static int partial_maximum(CSOUND *csound,P_MAXIMUM *p)
 {
-    int n = csound->ksmps, flag = (int) *p->imaxflag;
+    int n, nsmps = csound->ksmps, flag = (int) *p->imaxflag;
     MYFLT *a = p->asig;
     MYFLT max = p->max;
     switch(flag) {
     case 0: /* absolute maximum */
-      do {
+      for (n=0; n<nsmps; n++) {
         MYFLT temp;
-        if ((temp = (MYFLT) fabs(*a++)) > max) max = temp;
-      } while (--n);
+        if ((temp = FABS(a[n])) > max) max = temp;
+      }
       if (max > p->max) p->max = max;
       break;
     case 1: /* actual maximum */
-      do {
-        if (*a > max) max = *a;
-        ++a;
-      } while (--n);
+      for (n=0; n<nsmps; n++) {
+        if (a[n] > max) max = a[n];
+      }
       if (max > p->max) p->max = max;
       break;
     case 2: /* actual minimum */
-      do {
-        if (*a < max) max = *a;
-        ++a;
-      } while (--n);
+      for (n=0; n<nsmps; n++) {
+        if (a[n] < max) max = a[n];
+      }
       if (max < p->max) p->max = max;
       break;
     case 3: { /* average */
         MYFLT temp = FL(0.0);
         p->counter += n;
-        do {
-          temp += *a++;
-        } while (--n);
+        for (n=0; n<nsmps; n++) {
+          temp += a[n];
+        }
         p->max += temp;
       }
       break;
@@ -818,9 +846,9 @@ static OENTRY localops[] = {
   { "tb13.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab13_i_tmp   },
   { "tb14.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab14_i_tmp   },
   { "tb15.i",     S(FASTB), 1,    "i",     "i",    (SUBR) tab15_i_tmp   },
-  { "nlalp", S(NLALP),      5,    "a",    "akkoo",
+  { "nlalp",      S(NLALP), 5,    "a",     "akkoo",
                             (SUBR) nlalp_set, NULL, (SUBR) nlalp   },
-  { "adsynt2",S(ADSYNT2),   5,    "a",  "kkiiiio",
+  { "adsynt2",S(ADSYNT2),   5,    "a",     "kkiiiio",
                             (SUBR) adsynt2_set, NULL, (SUBR)adsynt2 },
   { "exitnow",S(EXITNOW),   1,    "",  "", (SUBR) exitnow, NULL, NULL },
 /* { "zr_i",  S(ZKR),     1,  "i",  "i",  (SUBR)zread, NULL, NULL}, */
