@@ -1,3 +1,4 @@
+
 /*
     pitch.c:
 
@@ -34,7 +35,7 @@
 #define PLAYING   2
 #define LOGTWO    (0.69314718055994530942)
 
-static int32 MYFLOOR(MYFLT x) {
+static inline int32 MYFLOOR(MYFLT x) {
   if (x >= 0.0) {
     return (int32) x;
   } else {
@@ -72,10 +73,10 @@ int pitchset(CSOUND *csound, PITCH *p)  /* pitch - uses spectra technology */
                                 /* Initialise spectrum */
     /* for mac roundoff */
     p->timcount = (int)(csound->ekr * *p->iprd + FL(0.001));
-    nocts = (int)*p->iocts; if (nocts<=0) nocts = 6;
-    nfreqs = (int)*p->ifrqs; if (nfreqs<=0) nfreqs = 12;
+    nocts = (int)*p->iocts; if (UNLIKELY(nocts<=0)) nocts = 6;
+    nfreqs = (int)*p->ifrqs; if (UNLIKELY(nfreqs<=0)) nfreqs = 12;
     ncoefs = nocts * nfreqs;
-    Q = *p->iq; if (Q<=FL(0.0)) Q = FL(15.0);
+    Q = *p->iq; if (UNLIKELY(Q<=FL(0.0))) Q = FL(15.0);
 
     if (UNLIKELY(p->timcount <= 0))
       return csound->InitError(csound, Str("illegal iprd"));
@@ -173,12 +174,12 @@ int pitchset(CSOUND *csound, PITCH *p)  /* pitch - uses spectra technology */
       p->fundp = (MYFLT *) p->wfund.auxch.auxp;
       p->winpts = npts;
     }
-    if (*p->inptls<=FL(0.0)) nptls = 4;
+    if (UNLIKELY(*p->inptls<=FL(0.0))) nptls = 4;
     else nptls = (int32)*p->inptls;
     if (UNLIKELY(nptls > MAXPTL)) {
       return csound->InitError(csound, Str("illegal no of partials"));
     }
-    if (*p->irolloff<=FL(0.0)) p->rolloff = FL(0.6);
+    if (UNLIKELY(*p->irolloff<=FL(0.0))) p->rolloff = FL(0.6);
     else p->rolloff = *p->irolloff;
     p->nptls = nptls;        /* number, whether all or odd */
     ptlmax = nptls;
@@ -213,7 +214,7 @@ int pitchset(CSOUND *csound, PITCH *p)  /* pitch - uses spectra technology */
     fundp = p->fundp;
     fendp = fundp + specp->npts;
     if (flop < fundp) flop = fundp;
-    if (fhip > fendp) fhip = fendp;
+    if (UNLIKELY(fhip > fendp)) fhip = fendp;
     if (UNLIKELY(flop >= fhip)) {                 /* chk hi-lo range valid */
       return csound->InitError(csound, Str("illegal lo-hi values"));
     }
@@ -288,7 +289,7 @@ int pitch(CSOUND *csound, PITCH *p)
     kvar = SQRT((MYFLT)q);       /* End of spectrum part */
 
     specp = &p->wsig;
-    if ((--p->scountdown)) goto nxt;      /* if not yet time for new spec  */
+    if (LIKELY((--p->scountdown))) goto nxt;  /* if not yet time for new spec  */
     p->scountdown = p->timcount;          /* else reset counter & proceed: */
     downp = &p->downsig;
     nocts = downp->nocts;
@@ -384,7 +385,7 @@ int pitch(CSOUND *csound, PITCH *p)
       }
       fp = flop;                               /* now srch fbins for peak */
       for (fmaxp = fp, fmax = *fp; ++fp<fhip; )
-        if (*fp > fmax) {
+        if (UNLIKELY(*fp > fmax)) {
           fmax = *fp;
           fmaxp = fp;
         }
@@ -421,7 +422,7 @@ int pitch(CSOUND *csound, PITCH *p)
         if ((absdiff = kval - p->kvalsav) < FL(0.0))
           absdiff = -absdiff;
         confirms = (int)(absdiff * p->confact); /* get interval dependency  */
-        if (p->jmpcount < confirms) {
+        if (UNLIKELY(p->jmpcount < confirms)) {
           p->jmpcount += 1;               /* if not enough confirms,  */
           goto output;                    /*    must wait some more   */
         } else {
@@ -499,7 +500,7 @@ typedef struct {
 static void initClockStruct(CSOUND *csound, void **p)
 {
     *p = csound->QueryGlobalVariable(csound, "readClock::counters");
-    if (*p == NULL) {
+    if (UNLIKELY(*p == NULL)) {
       csound->CreateGlobalVariable(csound, "readClock::counters",
                                            sizeof(CPU_CLOCK));
       *p = csound->QueryGlobalVariable(csound, "readClock::counters");
@@ -509,7 +510,7 @@ static void initClockStruct(CSOUND *csound, void **p)
 
 static inline CPU_CLOCK *getClockStruct(CSOUND *csound, void **p)
 {
-    if (*p == NULL)
+    if (UNLIKELY(*p == NULL))
       initClockStruct(csound, p);
     return (CPU_CLOCK*) (*p);
 }
@@ -565,7 +566,7 @@ int adsyntset(CSOUND *csound, ADSYNT *p)
 {
     FUNC    *ftp;
     int     count;
-    int32    *lphs;
+    int32   *lphs;
 
     p->inerr = 0;
 
@@ -633,7 +634,7 @@ int adsynt(CSOUND *csound, ADSYNT *p)
     MYFLT   amp0, amp, cps0, cps;
     int32    phs, inc, lobits;
     int32    *lphs;
-    int     n, nsmps = csound->ksmps, count;
+    int     n, nsmps = csound->ksmps, c, count;
 
     if (UNLIKELY(p->inerr)) {
       return csound->PerfError(csound, Str("adsynt: not initialised"));
@@ -654,18 +655,18 @@ int adsynt(CSOUND *csound, ADSYNT *p)
     ar = p->sr;
     memset(ar, 0, nsmps*sizeof(MYFLT));
 
-    do {
-      amp = *amptbl++ * amp0;
-      cps = *freqtbl++ * cps0;
+    for (c=0; c<count; c++) {
+      amp = amptbl[c] * amp0;
+      cps = freqtbl[c] * cps0;
       inc = (int32) (cps * csound->sicvt);
-      phs = *lphs;
+      phs = lphs[c];
       for (n=0; n<nsmps; n++) {
         ar[n] += *(ftbl + (phs >> lobits)) * amp;
         phs += inc;
         phs &= PHMASK;
       }
-      *lphs++ = phs;
-    } while (--count);
+      lphs[c] = phs;
+    }
     return OK;
 }
 
@@ -676,11 +677,11 @@ int hsboscset(CSOUND *csound, HSBOSC *p)
 
     if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
-      if (*p->ioctcnt < 2)
+      if (UNLIKELY(*p->ioctcnt < 2))
         octcnt = 3;
       else
         octcnt = (int)*p->ioctcnt;
-      if (octcnt > 10)
+      if (UNLIKELY(octcnt > 10))
         octcnt = 10;
       p->octcnt = octcnt;
       if (*p->iphs >= 0) {
@@ -784,7 +785,7 @@ int pitchamdfset(CSOUND *csound, PITCHAMDF *p)
     }
     else {
       downsamp = (int)MYFLT2LONG(downs);
-      if (downsamp < 1)
+      if (UNLIKELY(downsamp < 1))
         downsamp = 1;
       srate = csound->esr / (MYFLT)downsamp;
       upsamp = 0;
@@ -844,7 +845,7 @@ int pitchamdfset(CSOUND *csound, PITCHAMDF *p)
     if (*p->imedi < 1)
       p->medisize = 0;
     else
-      p->medisize = (int)(*p->imedi+FL(0.5))*2+1;
+      p->medisize = (int)MYFLT2LONG(*p->imedi)*2+1;
     p->mediptr = 0;
 
     if (p->medisize) {
@@ -1095,7 +1096,7 @@ int phsbnkset(CSOUND *csound, PHSORBNK *p)
     int    n, count;
     double  *curphs;
 
-    count = (int)(*p->icnt + FL(0.5));
+    count = (int)MYFLT2LONG(*p->icnt);
     if (UNLIKELY(count < 2))
       count = 2;
 
@@ -1344,7 +1345,7 @@ int GardnerPink_init(CSOUND *csound, PINKISH *p)
       p->grd_NumRows = 20;
       /* Warn if user tried but failed to give sensible number */
       if (UNLIKELY(*p->iparam1 != FL(0.0)))
-        csound->Message(csound, Str("pinkish: Gardner method requires 4-%d bands. "
+        csound->Warning(csound, Str("pinkish: Gardner method requires 4-%d bands. "
                                     "Default %ld substituted for %d.\n"),
                         GRD_MAX_RANDOM_ROWS, p->grd_NumRows,
                         (int) *p->iparam1);
@@ -1463,7 +1464,7 @@ int GardnerPink_perf(CSOUND *csound, PINKISH *p)
 double tanh(double);
 int clip_set(CSOUND *csound, CLIP *p)
 {
-    int meth = (int)(*p->imethod + FL(0.5));
+    int meth = (int)MYFLT2LONG(*p->imethod);
     p->meth = meth;
     p->arg = *p->iarg;
     p->lim = *p->limit;
@@ -1570,7 +1571,7 @@ int Fosckk(CSOUND *csound, XOSC *p)
     FUNC        *ftp;
     MYFLT       amp, *ar, *ftbl;
     MYFLT       inc, phs;
-    int nsmps = csound->ksmps;
+    int n, nsmps = csound->ksmps;
     int flen;
 
     ftp = p->ftp;
@@ -1583,11 +1584,11 @@ int Fosckk(CSOUND *csound, XOSC *p)
     inc = (*p->xcps * flen) * csound->onedsr;
     amp = *p->xamp;
     ar = p->sr;
-    do {
-      *ar++ = *(ftbl + (int)(phs)) * amp;
+    for (n=0; n<nsmps; n++) {
+      ar[n] = *(ftbl + (int)(phs)) * amp;
       phs += inc;
       if (phs>flen) phs -= flen;
-    } while (--nsmps);
+    }
     p->lphs = phs;
     return OK;
 }
@@ -1597,7 +1598,7 @@ int Foscak(CSOUND *csound, XOSC *p)
     FUNC        *ftp;
     MYFLT       *ampp, *ar, *ftbl;
     MYFLT       inc, phs;
-    int nsmps = csound->ksmps;
+    int n, nsmps = csound->ksmps;
     int flen;
 
     ftp = p->ftp;
@@ -1610,11 +1611,11 @@ int Foscak(CSOUND *csound, XOSC *p)
     inc = (*p->xcps * flen) * csound->onedsr;
     ampp = p->xamp;
     ar = p->sr;
-    do {
-      *ar++ = *(ftbl + (int)(phs)) * *ampp++;
+    for (n=0; n<nsmps; n++) {
+      ar[n] = *(ftbl + (int)(phs)) * ampp[n];
       phs += inc;
       if (phs>flen) phs -= flen;
-    } while (--nsmps);
+    }
     p->lphs = phs;
     return OK;
 }
@@ -1624,7 +1625,7 @@ int Foscka(CSOUND *csound, XOSC *p)
     FUNC        *ftp;
     MYFLT       amp, *ar, *cpsp, *ftbl;
     MYFLT       inc, phs;
-    int nsmps = csound->ksmps;
+    int n, nsmps = csound->ksmps;
     int flen;
 
     ftp = p->ftp;
@@ -1638,12 +1639,12 @@ int Foscka(CSOUND *csound, XOSC *p)
     amp = *p->xamp;
     cpsp = p->xcps;
     ar = p->sr;
-    do {
-      MYFLT inc = (*cpsp++ *flen * csound->onedsr);
-      *ar++ = *(ftbl + (int)(phs)) * amp;
+    for (n=0; n<nsmps; n++) {
+      MYFLT inc = (cpsp[n] *flen * csound->onedsr);
+      ar[n] = *(ftbl + (int)(phs)) * amp;
       phs += inc;
       if (phs>flen) phs -= flen;
-    } while (--nsmps);
+    }
     p->lphs = phs;
     return OK;
 }
@@ -1653,7 +1654,7 @@ int Foscaa(CSOUND *csound, XOSC *p)
     FUNC        *ftp;
     MYFLT       *ampp, *ar, *ftbl;
     MYFLT       phs;
-    int nsmps = csound->ksmps;
+    int n, nsmps = csound->ksmps;
     int flen;
 
     ftp = p->ftp;
@@ -1665,12 +1666,12 @@ int Foscaa(CSOUND *csound, XOSC *p)
     phs = p->lphs;
     ampp = p->xamp;
     ar = p->sr;
-    do {
-      MYFLT inc = (*p->xcps++ * flen) * csound->onedsr;
-      *ar++ = *(ftbl + (int)(phs)) * *ampp++;
+    for (n=0; n<nsmps; n++) {
+      MYFLT inc = (p->xcps[n] * flen) * csound->onedsr;
+      ar[n] = *(ftbl + (int)(phs)) * ampp[n];
       phs += inc;
       if (phs>flen) phs -= flen;
-    } while (--nsmps);
+    }
     p->lphs = phs;
     return OK;
 }
@@ -1683,7 +1684,7 @@ int Foscaa(CSOUND *csound, XOSC *p)
 
 int impulse_set(CSOUND *csound, IMPULSE *p)
 {
-    p->next = (int)(FL(0.5) + *p->offset * csound->esr);
+    p->next = (int)MYFLT2LONG(*p->offset * csound->esr);
     return OK;
 }
 
@@ -1692,7 +1693,7 @@ int impulse(CSOUND *csound, IMPULSE *p)
     int n, nsmps = csound->ksmps;
     int next = p->next;
     MYFLT *ar = p->ar;
-    if (next < csound->ksmps) {         /* Impulse in this frame */
+    if (UNLIKELY(next < csound->ksmps)) {          /* Impulse in this frame */
       MYFLT frq = *p->freq;     /* Freq at k-rate */
       int sfreq;                /* Converted to samples */
       if (frq == FL(0.0)) sfreq = INT_MAX; /* Zero means infinite */
@@ -1748,7 +1749,7 @@ int trnset(CSOUND *csound, TRANSEG *p)
       MYFLT alpha = **argp++;
       MYFLT nxtval = **argp++;
       MYFLT d = dur * csound->esr;
-      if ((segp->cnt = (int32)(d + FL(0.5))) < 0)
+      if ((segp->cnt = (int32)MYFLT2LONG(d)) < 0)
         segp->cnt = 0;
       else
         segp->cnt = (int32)(dur * csound->ekr);
@@ -1806,7 +1807,7 @@ int ktrnseg(CSOUND *csound, TRANSEG *p)
 int trnseg(CSOUND *csound, TRANSEG *p)
 {
     MYFLT  val, *rs = p->rslt;
-    int         nsmps = csound->ksmps;
+    int         n, nsmps = csound->ksmps;
     NSEG        *segp = p->cursegp;
     if (UNLIKELY(p->auxch.auxp==NULL)) {
       return csound->PerfError(csound, Str("transeg: not initialised (arate)\n"));
@@ -1831,25 +1832,25 @@ int trnseg(CSOUND *csound, TRANSEG *p)
         p->curval = val;
       }
       if (p->alpha == FL(0.0)) {
-        do {
-          *rs++ = val;
+        for (n=0; n<nsmps; n++) {
+          rs[n] = val;
           val += p->curinc;
-        } while (--nsmps);
+        }
       }
       else {
-        do {
-          *rs++ = val;
+        for (n=0; n<nsmps; n++) {
+          rs[n] = val;
           p->curx += p->alpha;
           val = segp->val + p->curinc *
             (FL(1.0) - EXP(p->curx));
-        } while (--nsmps);
+        }
       }
       p->curval = val;
       return OK;
 putk:
-      do {
-        *rs++ = val;
-      } while (--nsmps);
+      for (n=0; n<nsmps; n++) {
+        rs[n] = val;
+      }
     }
     return OK;
 }
@@ -1955,7 +1956,7 @@ int ktrnsegr(CSOUND *csound, TRANSEG *p)
 int trnsegr(CSOUND *csound, TRANSEG *p)
 {
     MYFLT  val, *rs = p->rslt;
-    int         nsmps = csound->ksmps;
+    int         n, nsmps = csound->ksmps;
     NSEG        *segp = p->cursegp;
     if (UNLIKELY(p->auxch.auxp==NULL)) {
       return csound->PerfError(csound, Str("transeg: not initialised (arate)\n"));
@@ -1989,25 +1990,25 @@ int trnsegr(CSOUND *csound, TRANSEG *p)
         p->curval = val;
       }
       if (p->alpha == FL(0.0)) {
-        do {
-          *rs++ = val;
+        for (n=0; n<nsmps; n++) {
+          rs[n] = val;
           val += p->curinc;
-        } while (--nsmps);
+        }
       }
       else {
-        do {
-          *rs++ = val;
+        for (n=0; n<nsmps; n++) {
+          rs[n] = val;
           p->curx += p->alpha;
           val = segp->val + p->curinc *
             (FL(1.0) - EXP(p->curx));
-        } while (--nsmps);
+        }
       }
       p->curval = val;
       return OK;
 putk:
-      do {
-        *rs++ = val;
-      } while (--nsmps);
+      for (n=0; n<nsmps; n++) {
+        rs[n] = val;
+      }
     }
     return OK;
 }
@@ -2026,7 +2027,7 @@ int varicolset(CSOUND *csound, VARI *p)
 
 int varicol(CSOUND *csound, VARI *p)
 {
-    int         nsmps = csound->ksmps;
+    int         n, nsmps = csound->ksmps;
     MYFLT       beta = *p->beta;
     MYFLT       sq1mb2 = p->sq1mb2;
     MYFLT       lastx = p->last;
@@ -2041,12 +2042,12 @@ int varicol(CSOUND *csound, VARI *p)
        ampmod = p->ampmod = FL(0.785)/(FL(1.0)+p->lastbeta);
     }
 
-    do {
+    for (n=0; n<nsmps; n++) {
       MYFLT rnd = FL(2.0) * (MYFLT) rand_31(csound) / FL(2147483645) - FL(1.0);
       lastx = lastx * beta + sq1mb2 * rnd;
-      *rslt++ = lastx * *kamp * ampmod;
+      rslt[n] = lastx * *kamp * ampmod;
       kamp += ampinc;
-    } while (--nsmps);
+    }
     p->last = lastx;
     return OK;
 }
@@ -2185,3 +2186,100 @@ int waveset(CSOUND *csound, BARRI *p)
     return OK;
 }
 
+int medfiltset(CSOUND *csound, MEDFILT *p)
+{
+    int maxwind = (int)MYFLT2LONG(*p->imaxsize);
+    int auxsize = 2*sizeof(MYFLT)*maxwind;
+    p->ind = 0;
+    p->maxwind = maxwind;
+
+    if (p->b.auxp==NULL || p->b.size < (size_t)auxsize)
+      csound->AuxAlloc(csound, (size_t)auxsize, &p->b);
+    else
+      if (*p->iskip!=FL(0.0)) memset(p->b.auxp, 0, auxsize);
+    p->buff = (MYFLT*)p->b.auxp;
+    p->med = &(p->buff[maxwind]);
+    return OK;
+}
+
+int medfilt(CSOUND *csound, MEDFILT *p)
+{
+    MYFLT *aout = p->ans;
+    MYFLT *asig = p->asig;
+    MYFLT *buffer = p->buff;
+    MYFLT *med = p->med;
+    int maxwind = p->maxwind;
+    int kwind = MYFLT2LONG(*p->kwind);
+    int index = p->ind;
+    int n, nsmps=csound->ksmps;
+    if (UNLIKELY(p->b.auxp==NULL)) {
+      return csound->PerfError(csound, Str("median: not initialised (arate)\n"));
+    }
+    if (UNLIKELY(kwind > maxwind)) {
+      csound->Warning(csound,
+                      Str("median: window (%d)larger than maximum(%d); truncated"),
+                      kwind, maxwind);
+      kwind = maxwind;
+    }
+    for (n=0; n<nsmps; n++) {
+      MYFLT x = asig[n];
+      buffer[index++] = x;
+
+      if (kwind<=index) {        /* all in centre */
+        memcpy(&med[0], &buffer[index-kwind], kwind*sizeof(MYFLT));
+        /* printf("memcpy: 0 <- %d (%d)\n", index-kwind, kwind); */
+      }
+      else {                    /* or in two parts */
+        memcpy(&med[0], &buffer[0], index*sizeof(MYFLT));
+        /* printf("memcpy: 0 <- 0 (%d)\n", index); */
+        memcpy(&med[index], &buffer[maxwind+index-kwind],
+               (kwind-index)*sizeof(MYFLT));
+        /* printf("memcpy: %d <- %d (%d)\n",
+           index, maxwind+index-kwind, kwind-index); */
+      }
+      /* { int i; */
+      /*   for (i=0; i<8; i++) printf(" %f", buffer[i]); */
+      /*   printf("\n\t:"); */
+      /*   for (i=0; i<5; i++) printf(" %f", med[i]); */
+      /*   printf("\n"); */
+      /* } */
+      aout[n] = medianvalue(kwind, med-1); /* -1 as should point below data */
+      /* printf("%d/$%d: %f -> %f\n", n, index-1, x, aout[n]); */
+      if (index>=maxwind) index = 0;
+    }
+    p->ind = index;
+    return OK;
+}
+
+int kmedfilt(CSOUND *csound, MEDFILT *p)
+{
+    MYFLT *buffer = p->buff;
+    MYFLT *med = p->med;
+    MYFLT x = *p->asig;
+    int maxwind = p->maxwind;
+    int kwind = MYFLT2LONG(*p->kwind);
+    int index = p->ind;
+    int n, nsmps=csound->ksmps;
+    if (UNLIKELY(p->b.auxp==NULL)) {
+      return csound->PerfError(csound, Str("median: not initialised (krate)\n"));
+    }
+    if (UNLIKELY(kwind > maxwind)) {
+      csound->Warning(csound,
+                      Str("median: window (%d)larger than maximum(%d); truncated"),
+                      kwind, maxwind);
+      kwind = maxwind;
+    }
+    buffer[index++] = x;
+    if (kwind<=index) {        /* all in centre */
+      memcpy(&med[0], &buffer[index-kwind], kwind*sizeof(MYFLT));
+    }
+    else {                    /* or in two parts */
+      memcpy(&med[0], &buffer[0], index*sizeof(MYFLT));
+      memcpy(&med[index], &buffer[maxwind+index-kwind],
+             (kwind-index)*sizeof(MYFLT));
+    }
+    *p->ans = medianvalue(kwind, med-1); /* -1 as should point below data */
+    if (index>=maxwind) index = 0;
+    p->ind = index;
+    return OK;
+}
