@@ -91,7 +91,7 @@ int vbap_SIXTEEN_control(CSOUND *csound, VBAP_SIXTEEN *p)
 
     MYFLT tmp_gains[SIXTEEN],sum=FL(0.0);
     if (UNLIKELY(p->dim == 2 && fabs(*p->ele) > 0.0)) {
-      csound->Message(csound,Str("Warning: truncating elevation to 2-D plane\n"));
+      csound->Warning(csound,Str("Warning: truncating elevation to 2-D plane\n"));
       *p->ele = FL(0.0);
     }
 
@@ -223,7 +223,7 @@ int vbap_SIXTEEN_init(CSOUND *csound, VBAP_SIXTEEN *p)
 
     /* other initialization */
     if (UNLIKELY(p->dim == 2 && fabs(*p->ele) > 0.0)) {
-      csound->Message(csound,Str("Warning: truncating elevation to 2-D plane\n"));
+      csound->Warning(csound,Str("Warning: truncating elevation to 2-D plane\n"));
       *p->ele = FL(0.0);
     }
     p->ang_dir.azi    = (MYFLT) *p->azi;
@@ -292,7 +292,7 @@ int vbap_SIXTEEN_moving_control(CSOUND *csound, VBAP_SIXTEEN_MOVING *p)
     MYFLT coeff, angle;
     MYFLT tmp_gains[SIXTEEN],sum=FL(0.0);
     if (UNLIKELY(p->dim == 2 && fabs(p->ang_dir.ele) > 0.0)) {
-      csound->Message(csound,Str("Warning: truncating elevation to 2-D plane\n"));
+      csound->Warning(csound,Str("Warning: truncating elevation to 2-D plane\n"));
       p->ang_dir.ele = FL(0.0);
     }
     if (*p->spread <FL(0.0))
@@ -322,7 +322,7 @@ int vbap_SIXTEEN_moving_control(CSOUND *csound, VBAP_SIXTEEN_MOVING *p)
       if (*p->field_am >= 0.0 && p->dim == 2) /* point-to-point */
         if (UNLIKELY(fabs(fabs(*p->fld[p->next_fld] - *p->fld[p->curr_fld])
                           - 180.0) < 1.0))
-          csound->Message(csound,
+          csound->Warning(csound,
                           Str("Warning: Ambiguous transition 180 degrees.\n"));
     }
     if (*p->field_am >= FL(0.0)) { /* point-to-point */
