@@ -78,12 +78,21 @@
 #include "csoundCore.h"
 #include "csmodule.h"
 
-#if defined(LINUX) || defined(__MACH__)
+#if defined(__MACH__)
+#if defined(MAC_OS_X_VERSION_10_6) && (MAC_OS_X_VERSION_MIN_REQUIRED>=MAC_OS_X_VERSION_10_6)
+#define NEW_MACH_CODE
+#else
+#define OLD_MACH_CODE
+#endif
+#endif
+
+#if defined(LINUX) || defined(NEW_MACH_CODE)
 #include <dlfcn.h>
 #elif defined(WIN32)
 #include <windows.h>
 #endif
 
+
 #if defined(HAVE_DIRENT_H)
 #  include <dirent.h>
 #  if 0 && defined(__MACH__)
@@ -97,7 +106,7 @@ int             closedir(DIR*);
 #if defined(WIN32)
 #  include <io.h>
 #  include <direct.h>
-#elif defined (OLD_MACH_CODE) 
+#elif defined (OLD_MACH_CODE)
 #  define ERR_STR_LEN 255
 #  include <mach-o/dyld.h>
 #endif
@@ -135,7 +144,7 @@ static  const   char    *plugindir64_envvar = "OPCODEDIR64";
 #  ifndef CS_DEFAULT_PLUGINDIR
 #    ifndef USE_DOUBLE
 #      define CS_DEFAULT_PLUGINDIR  "/usr/local/lib/csound/plugins"
-#    Else
+#    else
 #      define CS_DEFAULT_PLUGINDIR  "/usr/local/lib/csound/plugins64"
 #    endif
 #  endif
@@ -229,7 +238,7 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     if (UNLIKELY(fname[0] == '\0'))
       return CSOUND_ERROR;
     /* load library */
-/*  #if defined(LINUX) */
+/* #if defined(LINUX) */
 /*       printf("About to open library '%s'\n", libraryPath); */
 /* #endif */
     err = csound->OpenLibrary(&h, libraryPath);
@@ -337,6 +346,33 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     return CSOUND_SUCCESS;
 }
 
+static int csoundCheckOpcodeDeny(CSOUND *csound, const char *fname)
+{
+    /* Check to see if the fname is on the do-not-load list */
+    char buff[256];
+    char *p, *deny;
+    char *list = getenv("CS_OMIT_LIBS");
+    /* printf("DEBUG %s(%d): check fname=%s\n", __FILE__, __LINE__, fname); */
+    /* printf("DEBUG %s(%d): list %s\n", __FILE__, __LINE__, list); */
+    if (list==NULL) return 0;
+    strcpy(buff, fname);
+    strrchr(buff, '.')[0] = '\0'; /* Remove .so etc */
+    p = strdup(list);
+    deny = strtok(p, ",");
+    /* printf("DEBUG %s(%d): check buff=%s\n", __FILE__, __LINE__, deny); */
+    while (deny) {
+      /* printf("DEBUG %s(%d): deny=%s\n", __FILE__, __LINE__, deny); */
+      if (strcmp(deny, buff)==0) {
+        free(p);
+        /* printf("DEBUG %s(%d): found\n", __FILE__, __LINE__); */
+        return 1;
+      }
+      deny = strtok(NULL, ",");
+    }
+    free(p);
+    /* printf("DEBUG %s(%d): not found\n", __FILE__, __LINE__); */
+    return 0;
+}
 #ifdef mac_classic
 /* The following code implements scanning of "OPCODEDIR"
    and auto-loading of plugins for MacOS 9 */
@@ -691,6 +727,11 @@ int csoundLoadModules(CSOUND *csound)
                                 fname);
         continue;
       }
+      /* printf("DEBUG %s(%d): possibly deny %s\n", __FILE__, __LINE__,fname); */
+      if (csoundCheckOpcodeDeny(csound, fname)) {
+        csound->Warning(csound, Str("Library %s omitted\n"), fname);
+        continue;
+      }
       if (csoundCheckOpcodePluginFile(csound, fname) != 0)
         continue;               /* skip file if marked for deferred loading */
       sprintf(buf, "%s%c%s", dname, DIRSEP, fname);
@@ -901,7 +942,7 @@ PUBLIC void *csoundGetLibrarySymbol(void *library, const char *procedureName)
     return (void*) GetProcAddress((HMODULE) library, procedureName);
 }
 
-#elif defined(LINUX) || defined (__MACH__)
+#elif defined(LINUX) || defined (NEW_MACH_CODE)
 
 PUBLIC int csoundOpenLibrary(void **library, const char *libraryPath)
 {
@@ -926,7 +967,7 @@ PUBLIC void *csoundGetLibrarySymbol(void *library, const char *procedureName)
     return (void*) dlsym(library, procedureName);
 }
 
-#elif defined (OLD_MACH_CODE) 
+#elif defined (OLD_MACH_CODE)
 
 /* Set and get the error string for use by dlerror */
 static const char *error(int setget, const char *str, ...)
