@@ -171,14 +171,12 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
         csound->FileClose(csound, fd);
       }
     }
-#ifdef BETA
-      if (csound->delayederrormessages) {
-        if (O->msglevel>8)
-          csound->Warning(csound, csound->delayederrormessages);
-        free(csound->delayederrormessages);
-        csound->delayederrormessages = NULL;
-      }
-#endif
+    if (csound->delayederrormessages) {
+      if (O->msglevel>8)
+        csound->Warning(csound, csound->delayederrormessages);
+      free(csound->delayederrormessages);
+      csound->delayederrormessages = NULL;
+    }
     /* check for CSD file */
     if (csound->orchname == NULL)
       dieu(csound, Str("no orchestra name"));
@@ -300,7 +298,6 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
         mfree(csound, fileDir);
     }
 
-
     csound->Message(csound, Str("orchname:  %s\n"), csound->orchname);
     if (csound->scorename != NULL)
       csound->Message(csound, Str("scorename: %s\n"), csound->scorename);
@@ -319,6 +316,8 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
       csound->Message(csound, "********************\n");
       csound->Message(csound, "* USING NEW PARSER *\n");
       csound->Message(csound, "********************\n");
+      cs_init_math_constants_macros(csound);
+      cs_init_omacros(csound, csound->omacros);
       new_orc_parser(csound);
     }
     else {
@@ -327,6 +326,12 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
 #else
     otran(csound);                  /* read orcfile, setup desblks & spaces */
 #endif
+#if defined(USE_OPENMP)
+    if (csound->oparms->numThreads > 1) {
+        omp_set_num_threads(csound->oparms->numThreads);
+        csound->Message(csound, "OpenMP enabled: requested %d threads.\n", csound->oparms->numThreads);
+    }
+#endif
 
     /* IV - Jan 28 2005 */
     print_benchmark_info(csound, Str("end of orchestra compile"));
@@ -404,32 +409,6 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv)
     if (O->Midioutname != NULL || O->FMidioutname != NULL)
       openMIDIout(csound);
 
-    /* SYY - Mar 28, 2007 - Multithreaded Csound */
-
-    if (O->numThreads > 1) {
-        int i;
-        THREADINFO *current = NULL;
-
-        csound->multiThreadedBarrier1 = csound->CreateBarrier(O->numThreads + 1);
-        csound->multiThreadedBarrier2 = csound->CreateBarrier(O->numThreads + 1);
-        csound->multiThreadedComplete = 0;
-
-        for(i = 0; i < O->numThreads; i++) {
-            THREADINFO *t = csound->Malloc(csound, sizeof(THREADINFO));
-
-            t->threadId = csound->CreateThread(&kperfThread, (void *)csound);
-            t->next = NULL;
-
-            if(current == NULL) {
-                csound->multiThreadedThreadInfo = t;
-            } else {
-                current->next = t;
-            }
-            current = t;
-        }
-
-    }
-
     return musmon(csound);
 }
 
