@@ -35,7 +35,7 @@ See Alissa S. Crans, Thomas M. Fiore, and Raymon Satyendra,
 _Musical Actions of Dihedral Groups_, 2008 
 (arXiv:0711.1873v2).
 
-You can do a plain old transpositions
+You can do plain old transpositions
 by pressing 1, 2, 3, 4, 5, or 6.
 
 You can move each voice independently with the arrow keys: 
@@ -95,7 +95,7 @@ class Tonnetz(object):
         else:
             return self.R
     def sort(self, chord):
-        c = array(chord, Float).copy()
+        c = array(chord, 'd').copy()
         d = c[0:self.N]
         d.sort()
         c[0:self.N] = d
@@ -170,12 +170,12 @@ class Tonnetz(object):
         chord = tuple(self.keepInside(chord))
         return chord
     def tones(self, chord):
-        c = array(chord, Float).copy()
+        c = array(chord, 'd').copy()
         for i in xrange(self.N):
            c[i] = c[i] % self.tonesPerOctave
         return self.sort(c)
     def zeroFormModulus(self, chord):
-        c = array(chord, Float).copy()
+        c = array(chord, 'd').copy()
         for i in xrange(self.N):
            c[i] = c[i] % self.tonesPerOctave
         m = min(c)
@@ -324,7 +324,7 @@ class Tonnetz(object):
         for i in xrange(n):
             tail = l.pop(self.N - 1)
             l.insert(0, tail)
-        return array(l, Float)
+        return array(l, 'd')
     def invert(self, chord):
         chord = array(chord)
         c = chord[1:self.N].tolist()
@@ -458,7 +458,7 @@ class Tonnetz(object):
                 print chord,'keeps inside as',c
             return c
     def pitchclasses(self, chord):
-        c = array(chord, Float).copy()
+        c = array(chord, 'd').copy()
         for i in xrange(self.N):
             c[i] = self.pitchclass(chord[i])
         return c
@@ -526,7 +526,7 @@ class TonnetzModel(Tonnetz):
             for x in xrange(-self.R, self.R+1):
                 for y in xrange(x, x + self.R+1):
                     for z in xrange(x, x + self.R+1):
-                        trichord = array((x,y,z), Float)
+                        trichord = array((x,y,z), 'd')
                         trichord = tuple(self.sort(trichord))
                         if self.isPrism and self.isInsidePrism(trichord, self.R):
                             if trichord not in self.trichords:
@@ -826,10 +826,10 @@ endin
             ''')
         model.csound.setScore('''
             f1 0 8192 10 1
-            f0 600
+            f0 6000
             e
             ''')
-        model.csound.setCommand('csound -h -d -r 44100 -k 441 -m128 -b100 -B100 -odac6 temp.orc temp.sco')
+        model.csound.setCommand('csound -h -d -r 48000 -k 1000 -m128 -b1000 -B1000 -odac temp.orc temp.sco')
         model.csound.exportForPerformance()
         gc.disable()
         model.csound.compile()
@@ -863,7 +863,7 @@ if __name__ == '__main__':
     scene.width = 300 * 7
     scene.height = 300 * 5
     # Tonnetz for trichords
-    model = TonnetzModel(octaveCount=1, doCycle=False, doConnect=True, isPrism=True, enableCsound=True)
+    model = TonnetzModel(octaveCount=3, doCycle=False, doConnect=True, isPrism=True, enableCsound=True)
     # Ranged chord space
     #model = TonnetzModel(octaveCount=2, doCycle=False, doConnect=False, isCube=True, isPrism=False)
     # Tonnetz in ranged chord space
